`timescale 1ns / 1ps

module ALU(A, RegB,Imm, srcB, op, Out, zero, neg);

  input	[31:0] A;
  input	[31:0] RegB, Imm;
  input	[5:0] op;
  input srcB;
  output[31:0] Out;
  output    zero, neg;

  wire [31:0] A;
  wire [31:0] RegB, Imm;
  wire [31:0] B;
  wire srcB;
  reg  [31:0] result;
  reg  zero;
  reg  neg;
 
  // use srcB to select between RegB and Imm
  assign B = (srcB === 0)? RegB : Imm;
  
  // perform the ALU operation and set flags if the result is zero or negative

  always @(A or B or op) begin
    case (op)
//	  6'b000000: result = B << Inst[10:6];
      6'b000001: result = A + B;
      6'b000010: result = A - B;
      6'b000100: result = A & B;
      6'b001000: result = A | B;
      6'b010000: result = (A < B)? 1 : 0;  // Set on less than. Enables all branches using BEQ
	  6'b000011: result = {B[15:0], 16'h0000}; //LUI - Put the immed field into the 16 high order bits, 0 out lower bits
      6'b000101: result = A | {16'h0000, B[15:0]}; //ORI - Zero extend immed, OR with A
	  default:   result = 32'hxxxxxxxx;
    endcase
    zero = (result == 32'h00000000);
    neg  = result[31];    
  end

  // assign the computed result to the ALU output
  
  assign Out = result;

endmodule
