`timescale 1ns / 1ps
module DataMemory(address, write, read, readData, writeData);

  input	[31:0] address;
  input	write, read;
  output [31:0] readData;
  input [31:0] writeData;
  
  integer i;
  reg [1:0] ioBuffer[15:0];
  wire delayed_write;
  // slow down the write signal so that we have some setup time for the data coming in
  assign #10 delayed_write = write;
  
  // this is the actual memory array 
  
  reg [31:0] memory[0:255];

  // initialize memory with a program to compute the Nth Fibonacci number
  // input is stored in location 254 (00fe in hex), output will be in 255 (00ff)

  initial begin	   
	 for( i = 0; i < 256; i=i+1) begin
    	memory[i] = 32'h00000000;
	 end
	 memory[8'hfe] = 32'h00000003;			        // this is the input N
  end

  // the rising edge of the delayed_write control input causes data to be written into memory 
  // using delayed_write ensures some setup time, otherwise we'd be dependent on the order 
  // in which write and data change - if write happened first, we'd write the wrong value
  
  always @(posedge delayed_write) begin
	//Allow for writing to console if a certain memory location is referenced
	  if(address === 32'hffffffff) begin
		case(writeData[31:16])
			16'h0001: begin
				ioBuffer[0] = writeData[15:0]; // store disc #
			end
			16'h0002: begin
				ioBuffer[1] = writeData[15:0]; // store value of "from" peg
			end
			16'h0003: begin
				$display("Move Disc %d from peg %d to peg %d",ioBuffer[0],ioBuffer[1],writeData[15:0]);
			end
			default:
				$display("Write sequence invalid!");
		endcase
	end
	
	//Otherwise, just do the normal write
  	else
	  begin
//		$display("address=%h; data=%h; write=%h", address[7:0], writeData, write);
    	memory[address[7:0]] = writeData;
	  end
  end

  // data is always read if read is asserted, otherwise the data bus is tristated
	  
  assign readData = read ? memory[address[7:0]] : 32'hzzzzzzzz;

endmodule
