`timescale 1ns / 1ps

module RegFile(WriteData, Inst, regWrite, wrRegSel, RegA, RegB, clk);

  input	[31:0] WriteData;
  input	[31:0] Inst;
  input	regWrite;
  input [1:0] wrRegSel;
  output    [31:0] RegA;
  output    [31:0] RegB;
  input	clk;

  wire [4:0]  rs, rt, rd, wrReg;


  reg [31:0] RegFile[0:31];
  wire [31:0] RegA, RegB;

  // initialize register 31 to be a zero - we'll use this as a constant
  
  initial begin
    RegFile[0] = 0;	
	RegFile[5'h1D] = 32'hFFFFFFF0; // set the initial stack pointer
	RegFile[5'h1E] = 32'hFFFFFFF0; // set the initial frame pointer
	RegFile[5'h1F] = 32'hDEADBEEF; // set the initial return address
  end

  // pick out the indices of the three possible registers involved in an instruction 
  
  assign rs = Inst[25:21];
  assign rt = Inst[20:16];
  assign rd = Inst[15:11];
  
  // decide which register is the one that might be written to (depends on instruction)
  // 00 - rd
  // 01 - rt
  // 1X - hardwired to 31 for use in JAL 
	  
  assign wrReg = wrRegSel[1]? 5'b11111 : (wrRegSel[0] ? rd : rt);   
  
  // do two reads and, optionally, one write with the register file
  // read two registers and send them to the ALU
  assign 	RegA = RegFile[rs];
  assign	RegB = RegFile[rt];
    
  // write into a register (but not the register storing our constant 0)	
  always @(posedge clk) begin
    if (regWrite && (wrReg != 0)) begin
      RegFile[wrReg] = WriteData;
    end
  end

endmodule
